TBCHILD  RSECT
TBCHILD  RMODE  ANY
TBCHILD  AMODE  31
         COPY ASMMSP
         ASMDREG
         ASMMREL
* ********************************************************************
* TBCHILD - Child transaction program
*
* Copyright IBM(UK) Ltd 2014
* HLASM - High Level Assembler and Toolkit (5696-234)
*
* The purpose of this program is to link a number of storage areas
* into a shared linked list.
*
* ********************************************************************
C_EYE_LENGTH  EQU 16
C_POOL1_COUNT EQU 16               Amount of cells to create in pool1
C_POOL2_COUNT EQU 32               Amount of cells to create in pool2
C_LOOP_COUNT  EQU 1024
         PRINT OFF
         SYSSTATE ARCHLVL=1
         IEABRCX  DEFINE
         PRINT ON
*
* Prepare program execution environment
*
         SAVE (14,12),,*
         LARL  R9,STATICAREA
         USING (STATICAREA,STATICAREA_E),R9
         L     R10,0(,R1)           Point to input parms
         USING TB_MASTER,R10
*
* Ensure that the data we have been passed by the caller is correct
*
         CLC   TB_EYE,=CL(C_EYE_LENGTH)'TBMSTR MASTER>'
         BNE   E_IS_NOK
         CLC   TB_EYE_E,=CL(C_EYE_LENGTH)'<TBMSTR MASTER'
         BE    E_IS_OK
E_IS_NOK DC    0H
         DC    A(9)
E_IS_OK  DC    0H
         LA    R2,WS_LENGTH
         STORAGE OBTAIN,LENGTH=(R2),ADDR=(R3),COND=NO
         USING (WORKING_STORAGE,WORKING_STORAGE+WS_LENGTH),R3
         ST    R13,WS_SAVEA+4       save callers savearea
         LA    R13,WS_SAVEA         address my savearea
         ST    R13,WS_SAVEA+8       .. and save it
*
* Prepare working storage eyecatchers
*
         MVC   WS_EYE,=CL(C_EYE_LENGTH)'TBCHILD W/S->'
         MVC   WS_EYE_E,=CL(C_EYE_LENGTH)'TBCHILD W/S-<'
         MVC   WS_ISGENQ_S(S_ISGENQ_S_L),S_ISGENQ_S
         MVC   WS_WTO_ENDED(S_WTO_ENDED_L),S_WTO_ENDED
*
* Setup transaction support messages
*
         MVC   WS_TRAN_SUPPORT,TB_TRAN_SUPPORT
         MVC   WS_WTO_SUPPORT_Y(S_WTO_SUPPORT_YL),S_WTO_SUPPORT_Y
         MVC   WS_WTO_SUPPORT_C(S_WTO_SUPPORT_CL),S_WTO_SUPPORT_C
         MVC   WS_WTO_SUPPORT_N(S_WTO_SUPPORT_NL),S_WTO_SUPPORT_N
         MVC   WS_WTO_TABORT(S_WTO_TABORT_L),S_WTO_TABORT
         MVC   WS_WTO_DIS_TRAN(S_WTO_DIS_TRAN_L),S_WTO_DIS_TRAN
         MVC   WS_WTO_TEND(S_WTO_TEND_L),S_WTO_TEND
         MVC   WS_WTO_TNCS(S_WTO_TNCS_L),S_WTO_TNCS
         MVC   WS_WTO_TNCE(S_WTO_TNCE_L),S_WTO_TNCE
         MVC   WS_WTO_TCS(S_WTO_TCS_L),S_WTO_TCS
         MVC   WS_WTO_TCE(S_WTO_TCE_L),S_WTO_TCE
         MVC   WS_WTO_WAITS(S_WTO_WAITS_L),S_WTO_WAITS
         MVC   WS_WTO_WAITE(S_WTO_WAITE_L),S_WTO_WAITE
         MVC   WS_TRAN_EYE_S,=CL(C_EYE_LENGTH)'TRANSACTION>>>>>'
         MVC   WS_TRAN_EYE_E,=CL(C_EYE_LENGTH)'<<<<<TRANSACTION'
         XC    WS_TRAN_TDB(C_TDB_LEN),WS_TRAN_TDB   Clear the TDB
*
         USING PSA,0
         MVC   WS_PSATOLD,PSATOLD
         DROP  0
*
* Start of mainline code
*
         LA    R11,C_LOOP_COUNT
StartProcess DC 0H
         ST    R11,WS_LOOP_CNTR           Save loop counter
         XR    R5,R5
         LA    R7,WS_CELL_1P_GET
         LA    R8,C_POOL1_COUNT
*
* Start building a chain from the 1st subpool
*
GetPool1 DC    0H
         USING TB_SP1_PARM,R1
         CPOOL GET,                                                    X
               CPID=TB_CELL_ID_1
         ST    R1,0(,R7)
         LA    R7,4(,R7)
         AFI   R5,1
         MVC   SP1_EYE,=CL(C_EYE_LENGTH)'TBCHILD SP1->'
         MVC   SP1_TCB,WS_PSATOLD
         BCT   R8,GetPool1
         DROP  R1
         ST    R5,WS_CELL_COUNT_1
         XR    R5,R5
         LA    R7,WS_CELL_2P_GET
         LA    R8,C_POOL2_COUNT
*
* Start building a chain from the 2nd subpool
*
GetPool2 DC    0H
         USING TB_SP2_PARM,R1
         CPOOL GET,                                                    X
               CPID=TB_CELL_ID_2
         ST    R1,0(,R7)
         LA    R7,4(,R7)
         AFI   R5,1
         MVC   SP2_EYE,=CL(C_EYE_LENGTH)'TBCHILD SP2->'
         BCT   R8,GetPool2
         DROP  R1
         ST    R5,WS_CELL_COUNT_2
*
* Chain the cells together
*
         LA    R6,WS_CELL_1P_GET
         LA    R8,C_POOL1_COUNT
         BCTR  R8,0
         USING TB_SP1_PARM,R7
Chain1   DC    0H
         L     R7,0(,R6)         current cell
         LA    R6,4(,R6)         get next one
         ST    R6,SP1_F_PTR      save forward cell
         BCT   R8,Chain1         next cell
         L     R7,0(,R6)         last cell
         ST    R7,WS_CELL_1TAIL  save tail cell
         DROP  R7
         LA    R6,WS_CELL_2P_GET
         LA    R8,C_POOL2_COUNT-1
         USING TB_SP2_PARM,R7
Chain2   DC    0H
         L     R7,0(,R6)         current cell
         LA    R6,4(,R6)         get next one
         ST    R6,SP2_F_PTR      save forward cell
         BCT   R8,Chain2         next cell
         L     R7,0(,R6)         last cell
         ST    R7,WS_CELL_2TAIL  save tail cell
         DROP  R7
         LA    R6,WS_CELL_1P_GET
         LA    R7,WS_CELL_2P_GET
         USING TB_SP1_PARM,R6
         USING TB_SP2_PARM,R7
         L     R6,0(,R6)                        get new head
         L     R7,0(,R7)                        get new head
*
* Check for the presence of the transaction facilty
* If we can run a constrained transaction on the data rather
* than using a full ENQ, then do so.
*
         LR    R2,R9                       Free up register 9
         DROP  R9
         USING (STATICAREA,STATICAREA_E),2
         CLI   WS_TRAN_SUPPORT,C'C'
         IF    (EQ)                      Run constr transaction?
           WTO     MF=(E,WS_WTO_SUPPORT_C)
           J     CTRACT
         ENDIF ,
         CLI   WS_TRAN_SUPPORT,C'Y'      Run non-constr transaction?
         IF    (EQ)
           WTO     MF=(E,WS_WTO_SUPPORT_Y)
           J     NTRACT
         ENDIF ,
* If control flows through here, then run non-transactional code
GETENQ   WTO     MF=(E,WS_WTO_SUPPORT_N)
         ISGENQ REQUEST=OBTAIN,                                        X
               QNAME=S_ENQ_MAJOR_NAME,                                 X
               RNAME=S_ENQ_MINOR_NAME,                                 X
               RNAMELEN=S_ENQ_RNAME_LENGTH,                            X
               CONTROL=EXCLUSIVE,                                      X
               ENQTOKEN=WS_ENQ_TOKEN,                                  X
               SCOPE=STEP,                                             X
               MF=(E,WS_ISGENQ_S)
         J     COM_Q                 Skip transaction initialisation
*
* The transactional code will modify registers 6,7,8 and 9.
* Should the transaction abort for any reason, then these resgisters
* well be restored to their previous values.
REGS69   EQU   b'00011000'    Transaction GRSM for registers 6-9
*
* Start a non-constrained transaction
*
NTRACT   XR     R5,R5                Set register 5 as trns abort count
         LARL   R4,NTRACTS           Set address of transaction retry
         LARL   R11,GETENQ           Set address of non trans retry
NTRACTS  TBEGIN WS_TRAN_TDB,REGS69
         JNZ   TRAN_ABORT            Setup abort handler
         J     COM_Q                 Jump to common code
*
* Start a constrained transaction
*
CTRACT   TBEGINC 0,0
         JNZ   TRAN_ABORT            Setup abort handler
COM_Q    L     R8,TB_CELL_CHAIN_1    Get head 1 into working storage
         L     R9,TB_CELL_CHAIN_2    Get head 2 into working storage
         ST    R8,WS_CELL_1HEAD
         ST    R9,WS_CELL_2HEAD
         ST    R6,TB_CELL_CHAIN_1    save head
         ST    R7,TB_CELL_CHAIN_1    save head
         L     R6,WS_CELL_1TAIL      get last cell
         L     R7,WS_CELL_2TAIL      get last cell
         ST    R8,SP1_F_PTR               join chain
         ST    R9,SP2_F_PTR               join chain
         L     R8,TB_ENQ_COUNT_1
         L     R9,TB_DEQ_COUNT_1
         AFI   R8,1
         AFI   R9,1
         ST    R8,TB_ENQ_COUNT_1
         ST    R9,TB_DEQ_COUNT_1
         L     R8,TB_CELL_COUNT_1
         L     R9,TB_CELL_COUNT_2
         A     R8,WS_CELL_COUNT_1
         A     R9,WS_CELL_COUNT_2
         ST    R8,TB_CELL_COUNT_1
         ST    R9,TB_CELL_COUNT_2
*
* Check if we're running in a transaction and if we are not, then
* jump over the TEND instruction
*
         CLI   WS_TRAN_SUPPORT,C'N'
         IF    (NE)
* End the transaction and don't release an ENQ
           TEND
           WTO     MF=(E,WS_WTO_TEND)
         ELSE  ,
           DROP  R6
           DROP  R7
*
* Non-transactional ENQ release
*
           ISGENQ REQUEST=RELEASE,                                     X
               ENQTOKEN=WS_ENQ_TOKEN,                                  X
               MF=(E,WS_ISGENQ_S)
         ENDIF ,
*
* Simulate processing by waiting a short interval before continuing
*
         WTO     MF=(E,WS_WTO_WAITS)
MCONT    STIMER WAIT,TUINTVL=S_WAIT_TIME
         WTO     MF=(E,WS_WTO_WAITE)
*
* After the similulated processing wait is over, start releasing the
* storage and decoupling the chains.
*
*
* Prepare registers before iterating through
*
         MVC   WS_ISGENQ_S(S_ISGENQ_S_L),S_ISGENQ_S
         LA    R6,TB_CELL_CHAIN_1     Get top of chain
         LA    R8,C_POOL1_COUNT       number of entries to release
         LA    R7,WS_CELL_1P_REL      save pointers here
*
* Check the state of the transaction-execution facility and change
* into non-constrained transational mode if it is available.
*
         CLI   WS_TRAN_SUPPORT,C'N'
         IF    (NE)
           MVI   WS_TRAN_SUPPORT,C'Y'
         ENDIF ,
         CLI   WS_TRAN_SUPPORT,C'C'
         IF    (EQ)
           WTO     MF=(E,WS_WTO_TCS)
           J     CTRACTDQ
         ENDIF ,
         CLI   WS_TRAN_SUPPORT,C'Y'
         IF    (EQ)
           WTO     MF=(E,WS_WTO_TNCS)
           J     NTRACTDQ
         ENDIF ,
* Else run in non-transactional mode
DQENQ    ISGENQ REQUEST=OBTAIN,                                        X
               QNAME=S_ENQ_MAJOR_NAME,                                 X
               RNAME=S_ENQ_MINOR_NAME,                                 X
               RNAMELEN=S_ENQ_RNAME_LENGTH,                            X
               CONTROL=EXCLUSIVE,                                      X
               ENQTOKEN=WS_ENQ_TOKEN,                                  X
               SCOPE=STEP,                                             X
               MF=(E,WS_ISGENQ_S)
         J     COM_DEQ
*
* Start dequeuing chain 1 items
*
         USING TB_SP1_PARM,R6
COM_DEQ  DC    0H
         CLI   WS_TRAN_SUPPORT,C'N'   Are we running in non-trans mode?
         IF    (NE)
           CLI   WS_TRAN_SUPPORT,C'C'   Run in constr trans mode?
           IF    (NE)
NTRACTDQ     XR    R5,R5                No - clear trans count
             LARL  R4,NTRACTSQ          Prepare retry address
             LARL  11,DQENQ
NTRACTSQ     TBEGIN WS_TRAN_TDB,REGS69  Start unconstrained trans
             JNZ   TRAN_ABORT           Prepare abort handler
           ELSE ,
CTRACTDQ     TBEGINC 0,0
             JNZ   TRAN_ABORT
           ENDIF ,
         ENDIF ,
DQCOM    ST    R6,0(,R7)
         LA    R7,4(,R7)
         L     R6,SP1_F_PTR
         CLI   WS_TRAN_SUPPORT,C'N'
         IF    (NE)
           TEND
         ENDIF ,
         BCT   R8,COM_DEQ
         ST    R6,TB_CELL_CHAIN_1
         DROP  R6
         CLI   WS_TRAN_SUPPORT,C'N'
         IF    (NE)
           TEND
         ENDIF ,
*
* Start dequeuing chain 2 items
*
         L     R6,TB_CELL_CHAIN_2     get top of chain
         USING TB_SP2_PARM,R6
         LA    R8,C_POOL2_COUNT       number of entries to release
         LA    R7,WS_CELL_2P_REL save pointers here
UncPool2 DC    0H
         CLI   WS_TRAN_SUPPORT,C'N'   Non-transactional mode?
         IF    (NE)
           CLI   WS_TRAN_SUPPORT,C'C'   Constrained transactional mode?
           IF    (EQ)
             TBEGINC 0,0                Yes - start constrained trans
             JNZ   TRAN_ABORT
           ELSE ,
             TBEGIN WS_TRAN_TDB,REGS69  No - start unconstrained trans
             JNZ    TRAN_ABORT          Prepare abort handler
           ENDIF ,
         ENDIF ,
uncp2_c  ST    R6,0(,R7)
         LA    R7,4(,R7)
         L     R6,SP2_F_PTR
         CLI   WS_TRAN_SUPPORT,C'N'
         IF    (NE)
           TEND
           BCT   R8,UncPool2     Branch and reinitialise trans
         ELSE  ,
           BCT   R8,uncp2_c      Just loop back under the lock
         ENDIF ,
         ST    R6,TB_CELL_CHAIN_2
         DROP  R6
*
* If no transaction had taken place, then release the ENQ.
*
         CLI   WS_TRAN_SUPPORT,C'N'
         IF    (EQ)
RELDQENQ ISGENQ REQUEST=RELEASE,                                       X
               ENQTOKEN=WS_ENQ_TOKEN,                                  X
               MF=(E,WS_ISGENQ_S)
         ENDIF ,
*
* Now that the chains have been uncoupled from the main chain, the
* storage for them can be released.
*
MCONT2   LR    R9,R2                  Restore register 9 for Wrk Strg
         DROP  R2
         USING (STATICAREA,STATICAREA_E),R9
         LA    R7,WS_CELL_1P_GET
         LA    R8,C_POOL1_COUNT
FrePool1 DC    0H
         L     R6,0(,R7)
         CPOOL FREE,                                                   X
               CPID=TB_CELL_ID_1,                                      X
               CELL=(R6)
         LA    R7,4(,R7)
         BCT   R8,FrePool1
         LA    R7,WS_CELL_2P_GET
         LA    R8,C_POOL1_COUNT
FrePool2 DC    0H
         L     R6,0(,R7)
         CPOOL FREE,                                                   X
               CPID=TB_CELL_ID_2,                                      X
               CELL=(R6)
         LA    R7,4(,R7)
         BCT   R8,FrePool2
*
* If the process has more work to do, then loop around and start again
*
         L     R11,WS_LOOP_CNTR    reload loop counter
         BCT   R11,StartProcess    loop around
*
* *********************************************************************
* Mainline return point
* *********************************************************************
*
EXIT     DC    0H                    exit label
         WTO     MF=(E,WS_WTO_ENDED)
         L     R13,WS_SAVEA+4       restore callers savearea
         LA    R2,WS_LENGTH
         STORAGE RELEASE,LENGTH=(R2),ADDR=(R3),COND=NO
         RETURN (14,12),,RC=0     return
*
* *********************************************************************
* TRANSACTION ABORT ROUTINE
* *********************************************************************
*
* If the transaction fails to initialise or if it is aborted,
* then control will pass here to the TRAN_ABORT label.
*
* If the program was running in a non-constrained transaction, then
* we examine the Transaction Diagnostic Block (TDB)
*
* The number of times that a transaction attempts to redrive itself
* is determined by register 5.
*
* The transactional retry address is stored in register 4.
*
* The non-transactional retry address (once register 5 has reached a
* certain threshold), is stored in register 11.
*
TRAN_ABORT DS  0H
         WTO   MF=(E,WS_WTO_TABORT)
         CLI   WS_TRAN_SUPPORT,C'Y'     Non-constrained transaction?
         IF    (NE)
*          SOMETHING WENT SERIOUSLY WRONG!!
           LA    R3,WS_TRAN_TDB
           ABEND 20,DUMP
DISTRAN    MVI   WS_TRAN_SUPPORT,C'N'   Disable tran support for child
           WTO   MF=(E,WS_WTO_DIS_TRAN)
           BR    R11                    Redrive update logic
         ENDIF ,
* ### CODE FOR TDB EXAMINATION GOES HERE
* All diagnostics have been performed on the TDB
* Redrive the program logic wit processor assist for this child
* process
         AHI   R5,1                     Increase trns abort count
         CIJNL R5,20,DISTRAN            Failed 20 times?
         PPA   R5,0,1                   Perform proc assist
         BR    R4                       Redrive transaction
*
* ********************************************************************
* Child process static storage
* ********************************************************************
*
STATICAREA         DC 0D'0'           align storage
S_ISGENQ_S         ISGENQ MF=(L,S_,0D)
S_ISGENQ_S_L       EQU *-S_ISGENQ_S
S_ENQ_MAJOR_NAME   DC CL8'TBMSTR'
S_ENQ_MINOR_NAME   DC CL(32)'TBCHILD Enqueue Res'
S_ENQ_RNAME_LENGTH DC AL1(L'S_ENQ_MINOR_NAME)
S_WAIT_TIME DC F'1'      1 timer interval - approx 26 micro secs
*
* WTO macros
*
S_WTO_SUPPORT_Y  WTO 'Running in transactional mode',MF=L
S_WTO_SUPPORT_YL EQU *-S_WTO_SUPPORT_Y
S_WTO_SUPPORT_C  WTO 'Running in constrained trans mode',MF=L
S_WTO_SUPPORT_CL EQU *-S_WTO_SUPPORT_C
S_WTO_SUPPORT_N  WTO 'Running in non-transactional mode',MF=L
S_WTO_SUPPORT_NL EQU *-S_WTO_SUPPORT_N
S_WTO_TABORT     WTO 'Transaction aborted',MF=L
S_WTO_TABORT_L   EQU *-S_WTO_TABORT
S_WTO_DIS_TRAN   WTO 'Disabling transactional mode',MF=L
S_WTO_DIS_TRAN_L EQU *-S_WTO_DIS_TRAN
S_WTO_TEND       WTO 'Transaction ended',MF=L
S_WTO_TEND_L     EQU *-S_WTO_TEND
S_WTO_TNCS       WTO 'Non-constrained transaction started',MF=L
S_WTO_TNCS_L     EQU *-S_WTO_TNCS
S_WTO_TNCE       WTO 'Non-constrained transaction ended',MF=L
S_WTO_TNCE_L     EQU *-S_WTO_TNCE
S_WTO_TCS        WTO 'Con-constrained transaction started',MF=L
S_WTO_TCS_L      EQU *-S_WTO_TCS
S_WTO_TCE        WTO 'Con-constrained transaction ended',MF=L
S_WTO_TCE_L      EQU *-S_WTO_TCE
S_WTO_ENDED      WTO 'Child process ended',MF=L
S_WTO_ENDED_L    EQU *-S_WTO_ENDED
S_WTO_WAITS      WTO 'Child process starting wait',MF=L
S_WTO_WAITS_L    EQU *-S_WTO_WAITS
S_WTO_WAITE      WTO 'Child process finished wait',MF=L
S_WTO_WAITE_L    EQU *-S_WTO_WAITE
*
                 LTORG
STATICAREA_E     DC 0D'0'           align storage - end of static
*
* ********************************************************************
* Child process working storage
* ********************************************************************
*
WORKING_STORAGE DSECT ,           Program dynamic area
WS_EYE          DC CL(C_EYE_LENGTH)' '    Eye catcher
WS_SAVEA        DC 18F'0'                 save area
WS_ISGENQ_S     DC 0D,XL(S_ISGENQ_S_L)'00'  enq
WS_ENQ_TOKEN    DC CL(32)' '               enq token
WS_CELL_COUNT_1 DC F'0'
WS_CELL_COUNT_2 DC F'0'
WS_CELL_1P_GET  DC (C_POOL1_COUNT)A(0)     pointer to cell 1
WS_CELL_2P_GET  DC (C_POOL2_COUNT)A(0)     pointer to cell 2
WS_CELL_1P_REL  DC (C_POOL1_COUNT)A(0)     pointer to cell 1
WS_CELL_2P_REL  DC (C_POOL2_COUNT)A(0)     pointer to cell 2
WS_CELL_1HEAD   DC A(0)                    head of chain
WS_CELL_2HEAD   DC A(0)                    head of chain
WS_CELL_1TAIL   DC A(0)                    temp tail
WS_CELL_2TAIL   DC A(0)                    temp tail
WS_PSATOLD      DC A(0)                    tcb
WS_WTO_ENDED    DC 0D,XL(S_WTO_ENDED_L)'00'
WS_LOOP_CNTR    DC F'0'
*
* *********************************************************************
* TRANSACTIONAL-EXECUTION FACILITY DATA AREA
* *********************************************************************
*
WS_WTO_SUPPORT_Y  DC 0D,XL(S_WTO_SUPPORT_YL)'00'
WS_WTO_SUPPORT_C  DC 0D,XL(S_WTO_SUPPORT_CL)'00'
WS_WTO_SUPPORT_N  DC 0D,XL(S_WTO_SUPPORT_NL)'00'
WS_WTO_TABORT     DC 0D,XL(S_WTO_TABORT_L)'00'
WS_WTO_DIS_TRAN   DC 0D,XL(S_WTO_DIS_TRAN_L)'00'
WS_WTO_TEND       DC 0D,XL(S_WTO_TEND_L)'00'
WS_WTO_TNCS       DC 0D,XL(S_WTO_TNCS_L)'00'
WS_WTO_TNCE       DC 0D,XL(S_WTO_TNCE_L)'00'
WS_WTO_TCS        DC 0D,XL(S_WTO_TCS_L)'00'
WS_WTO_TCE        DC 0D,XL(S_WTO_TCE_L)'00'
WS_WTO_WAITS      DC 0D,XL(S_WTO_WAITS_L)'00'
WS_WTO_WAITE      DC 0D,XL(S_WTO_WAITE_L)'00'
WS_TRAN_EYE_S     DC CL(C_EYE_LENGTH)' ' Transaction support eyecatcher
WS_TRAN_SUPPORT   DC C'N'                Transaction support
*
WS_TRAN_TDB    DC 0D         Transaction Diagnostic block       +0
WS_TDB_FORM    DC XL.8'0'    TDB FORMAT                         +0
WS_TDB_FLAGS   DC XL.8'0'    TDB FLAGS                          +0
               DC 4C'0'    ! Reserved                           +0
WS_TDB_TND     DC XL.16'0'   TDB TND (Nesting depth)            +0
WS_TDB_TAC     DC XL8'0'     TDB TRANSACTION ABORT CODE         +8
WS_TDB_CONFL   DC XL8'0'     TDB CONFLICT LOCATION              +16
WS_TDB_ATIA    DC XL8'0'     TDB ABORTED TRANS INSTR ADDR       +24
WS_TDB_EAID    DC XL.8'0'    TDB EAID                           +32
WS_TDB_DXC     DC XL.8'0'    TDB DXC                            +32
               DC 2X'0'    ! Reserved                           +32
WS_TDB_PI_ID   DC XL.32'0'   TDB PROGRAM INTERRUPTION ID        +32
WS_TDB_EXCP_ID DC XL8'0'     TDB TRANSLATION EXCEPTION ID       +40
WS_TDB_BEA     DC XL8'0'     TDB BREAKING EVENT ADDRESS         +48
               DC 72X'0'   ! Reserved                           +56
WS_TDB_GPR     DC 120X'0'    TDB GPRs                           +128
C_TDB_LEN      EQU *-WS_TRAN_TDB
*
WS_TRAN_EYE_E  DC CL(C_EYE_LENGTH)' '    Transaction support eyecatcher
*
* *********************************************************************
* END OF TRANSACTIONAL-EXECUTION FACILITY DATA AREA
* *********************************************************************
*
WS_EYE_E       DC CL(C_EYE_LENGTH)' '    Eye catcher
               DC 0D              align end
WS_LENGTH      EQU *-WORKING_STORAGE WS Length
*
* Other DSECTs and COPYBOOKS
*
         COPY  TBDATA
         IHAPSA DSECT=YES,LIST=YES
         END   TBCHILD
